# ../addons/eventscripts/sourcesurvey/sourcesurvy
# Made by Steven Hartin

#########################
# DO NOT EDIT THIS FILE #
#   IF YOU DO, THINGS   #
#      COULD BREAK      #
#########################

import os
import random
import time
from sqlite3 import dbapi2 as sqlite

import es

import cfglib
import popuplib
import gamethread
import playerlib

import spawn

info          = es.AddonInfo()
info.name     = "Source Survey"
info.version  = "1.0.0"
info.url      = "http://addons.eventscripts.com/addons/view/sourcesurvey"
info.basename = "sourcesurvey"
info.author   = "Steven Hartin (freddukes)"

config = cfglib.AddonCFG( os.path.join ( es.getAddonPath(info.basename), "config.cfg") )
text   = config.text
cvar   = config.cvar

text(">>> ../addons/eventscripts/%s/config.cfg <<<" % info.basename )
text(">>> %s Version %s by %s <<<" % (info.name, info.version, info.author) )
text("")
captureType         = cvar("survey_captureType", "random", """The way in which people are asked to fill out surverys
// \"random\" = each person joining the server has a random chance of being asked to fill out a survey
// \"sequence\" = Every 1 in x people that joins the server will be asked to fill out a survey. """)
capturePercentage   = cvar("survey_capturePercentage", "10", """This is the chance that people will be asked to do your survey
// If survey_captureType is set to random, then this is the percentage that each person has of being picked
//   - for example, if this is set to 10, then each player has a 10% chance of being picked.
// If survey_captureType is set to sequence, then this means 1 person in this many will be asked...
//   - for example, if this is set to 10, then every 1 in 10 people to join will be asked.""")
specialAttributes   = cvar("survey_specialAttributes", "1", "Do players get certain attributes for finishing the survey? These are defined in ../spawn.py/")
attributeDecayLimit = cvar("survey_attributeDecayTimer", "10", "How long (in minutes) does the player receive special attributes for?")
spamDelay           = cvar("survey_spamDelay", "5", "How long must a user have to read a question before answering?\n// I recomend at least 5 seconds to stop users from pressing random buttons.")

config.write()
config.execute(True)

sourcesurvey = es.ServerVar('sourcesurvey', info.version, 'Source Survey Version - Made by Freddukes').makepublic()
    
def unload():
    cursor.close()
    connection.close()
    es.addons.unregisterSayFilter(sayFilter)
    
def player_spawn(ev):
    if not es.getplayerprop(ev['userid'], 'CBasePlayer.pl.deadflag'):
        """ Player is alive when player_spawn is ran, hence it's a normal event """
        surveys.playerSpawn(ev)
    """ 
        If execution reaches here, then it is the first time player_spawn activates
        This is ran when the player spawns for the first time but doesn't actually
        spawn...
    """
    surveys.testPlayer(int(ev['userid']))
    
class SurveyManager(object):
    count          = 0
    popups         = []
    benefits       = []
    players        = {}
    resultDir      = os.path.join( es.getAddonPath(info.basename), "results" )
    
    def playerSpawn(self, ev):
        """ Run the playerSpawn event in spawn.py to give the user special attributes """
        if ev['userid'] in self.benefits and int(specialAttributes):
            es.dbgmsg(1, 'Player %s is elegibal for benefits - sending them now' % es.getplayername(ev['userid']))
            spawn.playerSpawn(ev)
    
    def sendSurvey(self, userid):
        """ Add the user to the already surveyed list and the database, then send the popup """
        es.dbgmsg(1, 'SourceSurvey: Player %s has passed the test and is being surveyed' % es.getplayername(userid))
        steamid = playerlib.uniqueid(userid, True)
        cursor.execute("INSERT INTO players (Steamid) VALUES ('" + steamid + "')")
        alreadySurveyed.append(steamid)
        if userid not in self.players:
            self.players[userid] = {}
        self.players[userid]['time'] = time.time()
        popuplib.send("survey_info0", userid)
        connection.commit()
        es.tell(userid, '#multi', '#green[Source Survey] #default- #lightgreenYou have been chosen to complete a survey! ' + ("You will get bonuses if you complete it..." if int(specialAttributes) else " Please fill it out for the good of the community." ) )
    
    def sendNextQuestion(self, userid, choice, popupid):
        es.dbgmsg(1, 'SourceSurvey: Player %s answered a question, sending the next one now'%es.getplayername(userid))
        previousPopup = int( popupid.replace('survey_info', '') )
        es.dbgmsg(1, 'SourceSurvey: Player %s has just answered question %s' % (es.getplayername(userid), previousPopup + 1) )
        title = (popuplib.gPopups[popupid].l_titleformat['default'][:popuplib.gPopups[popupid].l_titleformat['default'].find('\n')])
        es.dbgmsg(1, 'SourceSurvey: Player has answered the question \"%s\" with answer \"%s\"'%(title, choice))
        es.dbgmsg(1, 'SourceSurvey: Amount of questions %s' % len(self.popups))
        if userid not in self.players:
            self.players[userid] = {}
        self.players[userid][title] = choice
        if len(self.popups) > previousPopup + 1:
            es.tell(userid, '#multi', '#green[Source Survey] #default- #lightgreenYou have just answered your %s/%s question'%(previousPopup + 1, len(self.popups)))
            es.dbgmsg(1, 'SourceSurvey: Not the end of the survey yet, send the next page.')
            self.popups[previousPopup + 1].send(userid)
            es.dbgmsg(1, 'SourceSurvey: Popup sent.')
        else:
            es.dbgmsg(1, 'SourceSurvey: The player has answered all questions.')
            self.writeToFile(userid)
            es.tell(userid, '#multi', '#green[Source Survey] #default- #lightgreenThank you, you have finished the survey!')
            if int(specialAttributes):
                es.dbgmsg(1, 'SourceSurvey: The player has been added to the benefit list.')
                self.benefits.append(str(userid))
                es.tell(userid, '#multi', '#green[Source Survey] #default- #lightgreenYou will now be rewarded for your hard work.')
    
    def testPlayer(self, userid):
        es.dbgmsg(1, 'SourceSurvey: Testing to see if player %s is legible for the survey'%es.getplayername(userid))
        if playerlib.uniqueid(userid, True) in alreadySurveyed:
            """ Do not even try to test a player that has already been surveyed """
            es.dbgmsg(1, 'SourceSurvey: Player already surveyed - returning')
            return
        if str(captureType) == "random":
            es.dbgmsg(1, 'SourceSurvey: Capture type set to random')
            rand = random.randint(1, 100)
            es.dbgmsg(1, 'SourceSurvey: Random number must be under %s to pass\n\t... Random number was: %s' % (capturePercentage, rand) )
            if not rand <= int(capturePercentage):
                return
            self.sendSurvey(userid)
        else:
            es.dbgmsg(1, 'SourceSurvey: Capture type set to pattern')
            self.count += 1
            es.dbgmsg(1, 'SourceSurvey: Count is %s, needs to reach %s before we send the survey.'% (self.count, capturePercentage) )
            if self.count >= int(capturePercentage):
                self.count = 0
                self.sendSurvey(userid)
                
    def quit(self, userid, choice = None, popupid = None):
        if choice is None and popupid is None:
            es.dbgmsg(1, 'SourceSurvey: User %s is quitting the survey early' % es.getplayername(userid))
            es.tell(userid, '#multi', '#green[Source Survey] #default- #lightgreenYou have quit the survey early' + (" therefore you will not get your bonuses" if int(specialAttributes) else ". We're sorry we could not interest you." ) )
            self.writeToFile(userid)
        else:
            if str(choice).startswith("answer_"):
                timeLeft = float(spamDelay) - (time.time() - self.players[userid]['time'])
                if timeLeft > 0:
                    es.tell(userid, '#multi', '#green[Source Survey] #default- #lightgreenYou cannot answer this question for another %s seconds' % (int(timeLeft) + 1) )
                    popuplib.send(popupid, userid)
                else:
                    self.players[userid]['time'] = time.time()
                    self.sendNextQuestion(userid, choice[7:], popupid)
            else:
                if int(choice) == 10:
                    es.dbgmsg(1, 'SourceSurvey: User %s is quitting the survey early' % es.getplayername(userid))
                    es.tell(userid, '#multi', '#green[Source Survey] #default- #lightgreenYou have quit the survey early' + (" therefore you will not get your bonuses" if int(specialAttributes) else ". We're sorry we could not interest you." ) )
                    self.writeToFile(userid)
                else:
                    popuplib.send(popupid, userid)
                
    def writeToFile(self, userid):
        es.dbgmsg(1, 'SourceSurvey: Writing results to file')
        if not os.path.isdir(self.resultDir):
            es.dbgmsg(1, 'SourceSurvey: ../results/ directory does not exist, creating now')
            os.mkdir(self.resultDir)
        year = time.strftime("%Y")
        if not os.path.isdir(os.path.join(self.resultDir, year) ):
            es.dbgmsg(1, 'SourceSurvey: ../results/%s directory does not exist, creating now'%year)
            os.mkdir(os.path.join(self.resultDir, year) )
        month = time.strftime("%B")
        if not os.path.isdir(os.path.join(self.resultDir, year, month) ):
            es.dbgmsg(1, 'SourceSurvey: ../results/%s/%s directory does not exist, creating now'%(year,month))
            os.mkdir(os.path.join(self.resultDir, year, month) )
        filePath = os.path.join(self.resultDir, year, month, time.strftime("%d") + ".txt" )
        text     = "\n\n"
        if not os.path.isfile(filePath):
            es.dbgmsg(1, 'SourceSurvey: ../results/%s/%s/%s.txt file does not exist, creating now'%(year,month,time.strftime("%d")) )
            text = text[2:]
            f = open(filePath, 'w')
        else:
            f = open(filePath, 'a')
        text += "-" * 50 + "\n"
        text += "Result taken at: %s\n" % (time.strftime("%H:%M:%S"))
        text += "\n"
        text += "Player Name: %s\n" % es.getplayername(userid)
        text += "Steamid: %s\n" % es.getplayersteamid(userid)
        text += "\n"
        text += "Questions Answered: %s\n" % (len(self.players[userid]) - 1)
        if len(self.players[userid]) == 1:
            text += "\tNo questions answered!\n"
        else:
            for title in self.players[userid]:
                if title == "time":
                    continue
                cursor.execute("SELECT Answers FROM results WHERE Question='" + title + "'")
                results       = cursor.fetchone()
                if results:
                    results = results[0]
                    splitresults  = results.split(":::SPLIT-FOR-QUESTION:::")
                    finalquestion = ""
                    for question in splitresults:
                        if question.startswith(self.players[userid][title]):
                            finalquestion = question
                            splitresults.remove(question)
                            break
                    finalquestion = finalquestion.split(":~:") # [answer, amount]
                    finalquestion[1] = str(int(finalquestion[1]) + 1)
                    finalquestion = ":~:".join(finalquestion)
                    splitresults.append(finalquestion)
                    splitresults = ":::SPLIT-FOR-QUESTION:::".join(splitresults)
                    cursor.execute("UPDATE results SET Answers='" + splitresults + "' WHERE Question='"+ title + "'")
                    text += "\t%s: %s\n" % (title, self.players[userid][title])
        text += "-" * 50
        f.write(text)
        f.close()
        connection.commit()
        es.dbgmsg(1, 'SourceSurvey: Results successfully saved!')
            
    def createQuestion(self, title, answers):
        if not answers or type(answers) not in (tuple, list):
            es.dbgmsg(1, "The answers to question \"%s\" are blank or are not of type \"tuple\" or \"list\". Please Insert answers"%title)
            return
        else:
            nextNumber = len(self.popups)
            p = popuplib.easymenu("survey_info%s"%nextNumber, '_popup_choice', self.sendNextQuestion)
            p.settitle(title + "\n")
            szAnswers = ""
            for answer in answers:
                szAnswers += "%s:~:0:::SPLIT-FOR-QUESTION:::"%answer
                p.addoption("answer_"+answer, answer)
            szAnswers = szAnswers[:-24]
            cursor.execute("SELECT * FROM results WHERE Question='" + title +"'")
            if cursor.fetchone() is None:
                cursor.execute("INSERT INTO results (Question, Answers) VALUES ('"+title+"','"+szAnswers+"')")
            else:
                cursor.execute("SELECT Answers FROM results WHERE Question='"+title+"'")
                newText = ""
                result = cursor.fetchone()[0]
                splitResults   = result.split(":::SPLIT-FOR-QUESTION:::")
                szAnswersSplit = szAnswers.split(":::SPLIT-FOR-QUESTION:::")
                szAnswersSplit = map(lambda x: x.split(":~:"), szAnswersSplit)
                splitResults   = map(lambda x: x.split(":~:"), splitResults) # [[answer,amount], [answer,amount]]
                for answer in szAnswersSplit:
                    if answer[0] not in map(lambda x: x[0], splitResults):
                        newText += "%s:~:0:::SPLIT-FOR-QUESTION:::" % answer[0]
                for answer in splitResults:
                    if answer[0] not in map(lambda x: x[0], szAnswersSplit):
                        continue
                    newText += "%s:~:%s:::SPLIT-FOR-QUESTION:::" % (answer[0], answer[1])
                newText = newText[:-24]
                cursor.execute("UPDATE results SET Answers='"+newText+"' WHERE Question='"+title+"'")
            connection.commit()
            p.menuselectfb = ""
            p.menuselect   = self.quit
            self.popups.append(p)
    
    def returnDictOfResults(self, title):
        """ Returns a dictionary with a key of the question and the value of the amount of votes """
        cursor.execute("SELECT Answers FROM results WHERE Question='" + title +"'")
        results = cursor.fetchone()[0]
        if results is None:
            return {}
        resultsDict = {}
        results = results.split(":::SPLIT-FOR-QUESTION:::")
        for result in results:
            result = result.split(":~:")
            resultsDict[result[0]] = result[1]
        return resultsDict
            
surveys = SurveyManager()

def surveyCommand():
    command = es.getargv(1).lower()
    if command == "create":
        if es.getargc() < 3:
            es.dbgmsg(0, "Invalid argument count: %s" % es.getargs() )
            return
        line = (" ".join(es.getargs().split()[1:])).split('"')[1::2]
        es.dbgmsg(1, '\n%s\nCreating a source survey:\nQuestion: %s'%("-" *50, line[0]))
        answers = map(lambda x: x.title(), line[1:])
        for answer in answers:
            es.dbgmsg(1, 'Answer: %s' % answer)
        es.dbgmsg(1, '-' * 50 + "\n")
        surveys.createQuestion(line[0], answers)
if not es.exists('command', 'survey'):
    es.regcmd('survey', 'sourcesurvey/surveyCommand', 'survey <create> [args]')
fPath = os.path.join(es.getAddonPath(info.basename), 'questions.cfg')
if not os.path.isfile(fPath):
    raise AttributeError, "Cannot find questions.cfg. Ensure it is at the following path\n\t%s"%fPath
es.server.queuecmd('es_xmexec ../addons/eventscripts/sourcesurvey/questions.cfg')

def sayFilter(userid, t, teamonly):
    text = t.strip('"')
    if text == '!results':
        popup = popuplib.easymenu('sourceserver_results', '_popup_choice', sendPlayerResultPopup)
        cursor.execute("SELECT Question FROM results")
        results = cursor.fetchall()
        for result in results:
            result = result[0]
            text = result
            if len(text) > 50:
                text = text[:47] + "..."
            popup.addoption(result, text)
        popup.settitle("Source Survey Results:\n -select a question-")
        popup.send(userid)
        return (0, "", 0)
    return (userid, t, teamonly)
es.addons.registerSayFilter(sayFilter)
    
def sendPlayerResultPopup(userid, choice, popupid):
    cursor.execute("SELECT Answers FROM results WHERE Question='"+choice+"'")
    result = cursor.fetchone()[0]
    result = result.split(":::SPLIT-FOR-QUESTION:::")
    result = map(lambda x: x.split(":~:"), result)
    popup  = popuplib.easymenu('sourceserver_results1', '_popup_choice', None)
    total  = sum(map(lambda x: int(x[1]), result))
    popup.settitle("Results for the question:\n%s\nTotal Voted: %s" % (choice, total) )
    for answer, amount in result:
        if not amount:
            percentage = "0.00"
        else:
            percentage = "%.2f" % (float(amount)/total*100)
        popup.addoption(None, "%s - %s (%s%%)"%(answer, amount, percentage), False)
    popup.submenu(10, 'sourceserver_results')
    popup.c_exitformat = "0. Back"
    popup.send(userid)

""" Execute the sqlite connections and get all the previous players who have already been surveyed """
alreadySurveyed = [] 
connection      = sqlite.connect( os.path.join( es.getAddonPath(info.basename), "players.db" ) )
cursor          = connection.cursor()
cursor.execute("CREATE TABLE IF NOT EXISTS players (Steamid TEXT PRIMARY KEY NOT NULL)")
cursor.execute("CREATE TABLE IF NOT EXISTS results (Question TEXT PRIMARY KEY NOT NULL, Answers TEXT NOT NULL)")
cursor.execute("SELECT Steamid FROM players")
players = cursor.fetchone()
if players is not None:
    for player in players:
        alreadySurveyed.append(player)